#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "terrain.h"

float
diamond_average (terrain_t * terrain, int i, int j, int sideLength)
{
  param_diamsquare_t *param;
  param = (param_diamsquare_t *) terrain->spec_param;
  int wrap = param->wrap;
  
  /* le centre du losange tant donn, cette fonction retourne la moyenne
     de ses 4 sommets  */

  if (i == 0) {
    if (wrap)
    return ((get_heightmap_value (terrain, 0, j - sideLength) +
	     get_heightmap_value (terrain, 0, j + sideLength) +
	     get_heightmap_value (terrain, terrain->size-1-sideLength,j) + 
       get_heightmap_value (terrain,sideLength, j)) / 4.0);
    else 
     return ((get_heightmap_value (terrain, 0, j - sideLength) +
	     get_heightmap_value (terrain, 0, j + sideLength) +
        get_heightmap_value (terrain,sideLength, j)) / 4.0);
  }
  
  else if (i == terrain->size - 1) {
    if (wrap)
    return ((get_heightmap_value (terrain, i, j - sideLength) +
	     get_heightmap_value (terrain, i, j + sideLength) +
	     get_heightmap_value (terrain, i - sideLength, j) +
	     get_heightmap_value (terrain, sideLength, j)) / 4.0); 
    else
       return ((get_heightmap_value (terrain, i, j - sideLength) +
	     get_heightmap_value (terrain, i, j + sideLength) +
	     get_heightmap_value (terrain, i - sideLength, j)) / 4.0);     
  }
  
  else if (j == 0) {
    if (wrap)
    return ((get_heightmap_value (terrain, i - sideLength, j) +
	     get_heightmap_value (terrain, i + sideLength, j) +
	     get_heightmap_value (terrain, i, j + sideLength) +
	     get_heightmap_value (terrain, i, terrain->size - 1 - j)) / 4.0);   
    else
      return ((get_heightmap_value (terrain, i - sideLength, j) +
	     get_heightmap_value (terrain, i + sideLength, j) +
	     get_heightmap_value (terrain, i, j + sideLength)) / 4.0); 
    
  }
  
  else if (j == terrain->size - 1) {
    if (wrap)
    return ((get_heightmap_value (terrain, i - sideLength, j) +
	     get_heightmap_value (terrain, i + sideLength, j) +
	     get_heightmap_value (terrain, i, j - sideLength) +
	     get_heightmap_value (terrain, i, sideLength)) / 4.0);
    else
      return ((get_heightmap_value (terrain, i - sideLength, j) +
	     get_heightmap_value (terrain, i + sideLength, j) +
	     get_heightmap_value (terrain, i, j - sideLength)) / 4.0);
  }
  
  else
    return ((get_heightmap_value (terrain, i - sideLength, j) +
	     get_heightmap_value (terrain, i + sideLength, j) +
	     get_heightmap_value (terrain, i, j - sideLength) +
	     get_heightmap_value (terrain, i, j + sideLength)) / 4.0);
}

float
square_average (terrain_t * terrain, int i, int j, int sideLength)
{
  /* le centre du carr tant donn, cette fonction retourne la moyenne
     de ses 4 sommets  */

  return ((get_heightmap_value (terrain, i - sideLength, j - sideLength) +
	   get_heightmap_value (terrain, i - sideLength, j + sideLength) +
	   get_heightmap_value (terrain, i + sideLength, j - sideLength) +
	   get_heightmap_value (terrain, i + sideLength,
				j + sideLength)) / 4.0);
}


void
gen_terrain_diamond (terrain_t * terrain)
{
  int i, j;
  int side_length;
  int odd;
  float range_reduction;
  int size = terrain->size - 1;
  float z;
  param_diamsquare_t *param;
  param = (param_diamsquare_t *) terrain->spec_param;
  float h = param->h;
  float range = param->factor;
  int wrap = param->wrap;

  /* the rate at which to reduce our random range after each iteration */
  range_reduction = (float) pow (2., -h);

  side_length = size >> 1;

  /* set the four corners to 0 */
  set_heightmap_value (terrain, 0, 0, 0);
  set_heightmap_value (terrain, 0, size, 0);
  set_heightmap_value (terrain, size, 0, 0);
  set_heightmap_value (terrain, size, size, 0);

  /* continue iterating until our sideLength is 0 */
  while (side_length) {

    /* diamond step */
    for (i = side_length; i < size; i += side_length) {
      for (j = side_length; j < size; j += side_length) {
	z =
	  range * randomRange (.5f) + square_average (terrain, i, j,
						      side_length);


	set_heightmap_value (terrain, i, j, z);
	if (z > terrain->max_height) {
	  terrain->max_height = z;
	}
	j += side_length;
      }
      i += side_length;
    }

    /* square step */
    odd = 0;
    for (i = 0; i < size; i += side_length) {
      odd = (odd == 0);
      for (j = 0; j < size; j += side_length) {
	if ((odd) && !j) {
	  j += side_length;
	}

	z = range *
	  randomRange (.5f) + diamond_average (terrain, i, j, side_length);

	set_heightmap_value (terrain, i, j, z);
	if (z > terrain->max_height)
	  terrain->max_height = z;

	/* wrap edges */
	if ((i == 0)&& (wrap))
	  set_heightmap_value (terrain, size, j,
			       get_heightmap_value (terrain, i, j));
	if ((j == 0) && (wrap))
	  set_heightmap_value (terrain, i, size,
			       get_heightmap_value (terrain, i, j));

	j += side_length;
      }
    }

    /* reduce random number range */
    range *= range_reduction;
    side_length >>= 1;
  }
  
  printf ("DIAMOND: max_height=%f\n", terrain->max_height);
}
