#ifndef __TERRAIN_H__
#define __TERRAIN_H__

/* differents types de terrains que l'on peut gnrer */
enum
{
  TERR_FLAT,        /* terrain plat  z=0, pour les tests */ 
  TERR_MIDDISP,     /* generation avec le midpoint displacement */
  TERR_DIAM,
  TERR_SPHERE,      /*  */
  TERR_HYPERB
};

/* diffrents filtres applicables */
enum
{ 
  TERR_FILTER_NOTHING,
  TERR_FILTER_FIR,
  TERR_FILTER_PUISS,
  TERR_FILTER_RIV,
  TERR_FILTER_CRAT,
  
  TERR_NUM_FILTERS
};

enum 
{
  FILTER1,
  FILTER2,
  FILTER3,
  FILTER4,
  
  NUM_EFFECTIVE_FILTERS 
};

/* color schemes */
enum
{
  TERR_COLOR_SUMMER,
  TERR_COLOR_MARS,
  TERR_COLOR_VENUS,
  
  TERR_NUM_COLORS
};

#define DEFAULT_TERR_ALGO TERR_FLAT
#define DEFAULT_TERR_COLOR TERR_COLOR_SUMMER
#define DEFAULT_TERR_SIZE 129
#define DEFAULT_TERR_SCALE 129
#define DEFAULT_TERR_H 0.93

typedef struct param_hyperb_t
{
  double power;
  float factor;
  int num_iter;
  int radius_max;
  int radius_min;
  int epsilon_type;
  
} param_hyperb_t;

typedef struct param_diamsquare_t
{
  float h; 
  float factor;
  int wrap;
  
} param_diamsquare_t;

typedef struct param_middisp_t
{
  float h; 
  float factor;
  
} param_middisp_t;


typedef struct filter_t
{
  int name;
  float param;
} filter_t;

typedef struct terrain_t
{
  int size; /* terrain de dimensions size x size */ 
 
  float *heightmap_orig; /* heightmap originale generee */
  float *heightmap; /* hightmap actuelle, i.e. deformee par les filtres */
  float max_height; /* hauteur maximale generee */
  
  unsigned int displaylist;
  void *spec_param; /* parametres specifiques */
  
  int algo;
  filter_t filters[NUM_EFFECTIVE_FILTERS];
  int color_scheme;
  
  int generated;
  
} terrain_t;

#include "common.h"

char init_terrain(terrain_t *terrain);
char gen_terrain(terrain_t *terrain);
void copy_orig_terr_to_effective (terrain_t *terrain);
void copy_effective_terr_to_orig (terrain_t *terrain);
char apply_filter (terrain_t * terrain, int filter, float param);

void gen_terrain_middisp(terrain_t *terrain);
void gen_terrain_diamond(terrain_t *terrain);
void gen_terrain_sphere(terrain_t *terrain);
void gen_terrain_hyperbole(terrain_t *terrain);

void TER_update_maxheight_value (terrain_t *terrain);

int TER_import_terrain (terrain_t * terrain, void *param, char *filename);
int TER_export_terrain (terrain_t * terrain, char *filename);
int TER_read_algo (char *filename);

char terr_import_raw(terrain_t *terrain, char * filename);
int TER_export_png_bw (terrain_t *terrain, char *filename);

#ifdef USE_BMP_EXPORT
int terr_export_bmp_bw (terrain_t *terrain, char *filename);
#endif /* USE_BMP_EXPORT */

#endif /* __TERRAIN_H__ */
